/***************************************************************************
 *                                                                         *
 *   KNetLoad is copyright (c) 1999-2000, Markus Gustavsson                *
 *                         (c) 2002, Ben Burton                            *
 *                         (c) 2004, Diego Petten                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "knetdock.h"
#include "knetload.h"
#include "knetproc.h"
#include "settings.h"
#include "interfaces.h"

#include <kaction.h>
#include <kconfigdialog.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kpopupmenu.h>
#include <qdir.h>
#include <kcombobox.h>

#define MAX_NET_DEV_LINE 512

KNetLoad::KNetLoad(QWidget *parent, const char *name) :
        StatPopup(parent, name) {
    // Create the raw network data reader.
    proc = new KNetProc();

// Stock device names are not i18n()ed since they're literal
// interface names.
#ifndef Q_OS_LINUX
    stockDevice.push_back("lo");    stockDeviceIcon.push_back("devlo");
    stockDevice.push_back("eth0");  stockDeviceIcon.push_back("deveth0");
    stockDevice.push_back("ppp0");  stockDeviceIcon.push_back("devppp0");
    stockDevice.push_back("ippp0"); stockDeviceIcon.push_back("devippp0");
#else
    if ( QDir::root().exists("/sys/class/net") )
    { // Exists /sys, 2.6 series kernel
        QDir sys("/sys/class/net");
        QStringList l = sys.entryList();
        for(QStringList::iterator it = l.begin(); it != l.end(); it++)
        {
            if ( (*it)[0] == '.' )
                continue;

            stockDevice.push_back( *it );
            if ( *it == "lo" )
                stockDeviceIcon.push_back("devlo");
            else if ( (*it).find("eth") == 0 )
                stockDeviceIcon.push_back("deveth0");
            else if ( (*it).find("ppp") == 0 )
                stockDeviceIcon.push_back("devppp0");
            else if ( (*it).find("ippp") == 0 )
                stockDeviceIcon.push_back("devippp0");
            else
                stockDeviceIcon.push_back("devother");
        }
    } else { // Doesn't exists, kernel 2.4 or earlier
        static FILE* fd;
        static char line[MAX_NET_DEV_LINE];
        static char* pos;
        static char* iface;


        if ((fd = fopen("/proc/net/dev", "r")) == 0)
            return;

        // Read the unwanted header lines.
        fgets(line, MAX_NET_DEV_LINE, fd);
        fgets(line, MAX_NET_DEV_LINE, fd);

        // Read through the remaining lines until we find all devices
        while (! feof(fd)) {
            fgets(line, MAX_NET_DEV_LINE, fd);

            // Find the interface name for this line.
            for (iface = line; *iface == ' '; iface++)
                ; // (skip initial spaces)
            for (pos = iface; *pos != ':' && *pos != 0; pos++)
                ; // (move to next ':' or end of string)
            if (*pos == 0)
                continue; // (was not ':')
            *pos = 0;

            // Now iface points to a null-terminated string containing the
            // interface name for this particular line.
            stockDevice.push_back( iface );
            if ( strncmp(iface, "lo", 2) == 0 )
                stockDeviceIcon.push_back("devlo");
            else if ( strncmp(iface, "eth", 3) == 0 )
                stockDeviceIcon.push_back("deveth0");
            else if ( strncmp(iface, "ppp", 3) == 0 )
                stockDeviceIcon.push_back("devppp0");
            else if ( strncmp(iface, "ippp", 4) == 0 )
                stockDeviceIcon.push_back("devippp0");
            else
                stockDeviceIcon.push_back("devother");
        }
        fclose(fd);
    }
#endif

    InterfacesConfig *ifcfg = new InterfacesConfig(0, "Interfaces");
    QStringList::iterator devit = stockDevice.begin(), xpmit = stockDeviceIcon.begin();
    for(; devit != stockDevice.end(); devit++, xpmit++)
    	ifcfg->kcfg_IfName->insertItem(KGlobal::iconLoader()->loadIcon(*xpmit, KIcon::Small), *devit);
    
    cfgdialog->addPage(ifcfg, i18n("Interfaces"), "network");

    // Set up actions and read the config file.
    setupActions();

    // Create system tray windows.
    dock[0] = new KNetDock(0, true, this); // In
    dock[1] = new KNetDock(1, false, this); // Out

    // Initialise the pop-up window.
    readPopupState();

    // Off we go!
    requestResize();
    if (Settings::enabled())
        startUpdates();
}

KNetLoad::~KNetLoad() {
    delete proc;
}

QString KNetLoad::dockName(int which) const {
    return (which == 0 ? i18n("In") : i18n("Out"));
}

void KNetLoad::takeReadingInternal() {
	proc->readLoad();
	
	// Rates in bits per second.
	bitRateIn = proc->recentBytesIn() * 8.0 /
		(((float) Settings::timeout()) / 1000.0);
	bitRateOut = proc->recentBytesOut() * 8.0 /
		(((float) Settings::timeout()) / 1000.0);

	// Convert to percentages.
	upper[0] = (int) (100.0 * bitRateIn / ((float) Settings::scaleIn()));
	upper[1] = (int) (100.0 * bitRateOut /
		((float) (Settings::scaleOut() ? Settings::scaleOut() : Settings::scaleIn() )));

	if (upper[0] < 0)
		upper[0] = 0;
	if (upper[1] < 0)
		upper[1] = 0;
	if (upper[0] > 100)
		upper[0] = 100;
	if (upper[1] > 100)
		upper[1] = 100;

	if (isVisible())
	{
		float currBitIn = (int)((bitRateIn  + 50) / 100) / 10.0;
		float currBitOut = (int)((bitRateOut  + 50) / 100) / 10.0;
		if ( Settings::speedUnit() == 0 ) // bits per second
		{
			fullReading = i18n(
				"Current In: %1 Kb/s, Total In: %2 MB.\n"
				"Current Out: %3 Kb/s, Total Out: %4 MB.")
				.arg(currBitIn).arg(proc->totalMbIn())
				.arg(currBitOut).arg(proc->totalMbOut());
		} else { // bytes per second
			currBitIn /= 8; currBitOut /= 8;
			
			fullReading = i18n(
				"Current In: %1 KB/s, Total In: %2 MB.\n"
				"Current Out: %3 KB/s, Total Out: %4 MB.")
				.arg(currBitIn).arg(proc->totalMbIn())
				.arg(currBitOut).arg(proc->totalMbOut());
		}
	}
}

void KNetLoad::reloadSettings()
{
	QString newDevice = Settings::ifName().stripWhiteSpace();
	if (newDevice.isEmpty())
	{
		KMessageBox::error(firstDock(),
			i18n("The device name cannot be empty."));
	} else {
		proc->setDevice(newDevice);
		return;
	}
	
	clearHistory();
	requestResize();
}

#include "knetload.moc"
